use chrono::{Duration, Local, NaiveDate};

// This interval doesn't feel right, either. The idea that I have a specific interval type for just
// NaiveDate is odd. This should be genericized, as should the iterator. Also, it shouldn't live
// here, but in utilities.
#[derive(Clone, Debug, PartialEq, Eq)]
pub struct DayInterval {
    pub start: NaiveDate,
    pub end: NaiveDate,
}

impl Default for DayInterval {
    fn default() -> Self {
        Self {
            start: (Local::now() - Duration::days(7)).date_naive(),
            end: Local::now().date_naive(),
        }
    }
}

impl DayInterval {
    pub fn days(&self) -> impl Iterator<Item = NaiveDate> {
        DayIterator {
            current: self.start,
            end: self.end,
        }
    }
}

struct DayIterator {
    current: NaiveDate,
    end: NaiveDate,
}

impl Iterator for DayIterator {
    type Item = NaiveDate;

    fn next(&mut self) -> Option<Self::Item> {
        if self.current <= self.end {
            let val = self.current;
            self.current += Duration::days(1);
            Some(val)
        } else {
            None
        }
    }
}
